package com.alterdekim.game.controller;

import com.alterdekim.game.component.GameServer;
import com.alterdekim.game.component.StartUpListener;
import com.alterdekim.game.controller.result.api.PreloaderResult;
import com.alterdekim.game.controller.result.api.PromotionBannerResult;
import com.alterdekim.game.controller.result.api.PromotionResult;
import com.alterdekim.game.entity.Preloader;
import com.alterdekim.game.entity.Promotion;
import com.alterdekim.game.entity.PromotionBanner;
import com.alterdekim.game.service.PreloaderService;
import com.alterdekim.game.service.PromotionBannerService;
import com.alterdekim.game.service.PromotionService;
import com.alterdekim.game.service.UserService;
import com.alterdekim.game.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/api")
public class ApiController {
    @Autowired
    private Environment env;

    @Autowired
    private StartUpListener startUpListener;

    @Autowired
    private GameServer gameServer;

    @Autowired
    private UserService userService;

    @Autowired
    private PreloaderService preloaderService;

    @Autowired
    private PromotionService promotionService;

    @Autowired
    private PromotionBannerService promotionBannerService;

    @GetMapping("/get_dashboard_logs")
    public ResponseEntity<byte[]> getLogs() {
        try {
            Path logFile = Paths.get(env.getProperty("logging.file.name"));
            RandomAccessFile file = new RandomAccessFile(logFile.toFile(), "r");
            int n = 2048;
            file.seek(logFile.toFile().length() - n);
            byte[] b = new byte[n];
            file.read(b, 0, n);
            return ResponseEntity.ok(b);
        } catch (IOException e) {
            log.error("getLogs error: {}", e.getMessage());
            return ResponseEntity.badRequest().build();
        }
    }

    @GetMapping("/get_uptime")
    public ResponseEntity<String> getUpTime() {
        return ResponseEntity.ok(
                DateUtils.prettyPrint(Duration.of(System.currentTimeMillis() - startUpListener.getStartTime(), ChronoUnit.MILLIS))
        );
    }

    @GetMapping("/get_players_online")
    public ResponseEntity<Integer> getPlayersOnline() {
        return ResponseEntity.ok(gameServer.getPlayersCount());
    }

    @GetMapping("/get_users_count")
    public ResponseEntity<Long> getUsersCount() {
        return ResponseEntity.ok(userService.getUsersCount());
    }

    @RequestMapping(value = "/get_preloaders", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<PreloaderResult>> getPreloaders() {
        return ResponseEntity.ok(preloaderService.getAll().stream().map(Preloader::toAPIResult).collect(Collectors.toList()));
    }

    @RequestMapping(value = "/get_promotions", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<PromotionResult>> getPromotions() {
        return ResponseEntity.ok(promotionService.getAll().stream().map(Promotion::toAPIResult).collect(Collectors.toList()));
    }

    @RequestMapping(value = "/get_promotion_banners", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<PromotionBannerResult>> getPromotionBanners() {
        return ResponseEntity.ok(promotionBannerService.getAll().stream().map(PromotionBanner::toAPIResult).collect(Collectors.toList()));
    }
}
